@@ -67,6 +67,11 @@ __authors__  = (
     'Charles Chen',
     'Hassaan Ali',
     'Dobrosław Żybort',
+    'David Fabijan',
+    'Sebastian Haas',
+    'Alexander Kirk',
+    'Erik Johnson',
+    'Keith Beckman',
 )
 
 __license__ = 'Public Domain'
